//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                   (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	Ranlux.cc:	--- implementation of Ranlux class
//
//	DESCRIPTION:
//		Implementation of Ranlux class - based on the Fortran
//		subroutine RANLUX in ranlux.f:
//		Subtract-and-borrow random number generator proposed by
//		Marsaglia and Zaman, implemented by F. James with the name
//		RCARRY in 1991, and later improved by Martin Luescher
//		in 1993 to produce "Luxury Pseudorandom Numbers".
//		Fortran 77 coded by F. James, 1993
//
//		M. Luscher, Computer Physics Communications  79 (1994) 100
//		F. James, Computer Physics Communications 79 (1994) 111
//
//	CHANGE HISTORY:
//		UL, 1999-05-27: 1.0.0: - first version
//		UL, 1999-06-08: 1.1.0: - added implementation if min()/max()
//
//	AUTHOR:
//		U. Lammers (UL), Thu May 27 16:15:23 MET DST 1999
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/utils/src/Ranlux.cc,v 1.3 2001/07/12 14:59:59 ulammers Exp $
//

#include <ctime>
#include <cstdlib>

#include "Ranlux.h"
#include "cfortran.h"
#include "common.h"

//
//	create wrappers for Fortran subroutines
//

//#define SLATEC_AS_MODULE  //  uncomment if SLATEC is a module

#if !defined(AS_MODULE)
	#if defined(F90MODPROC)
		#undef F90MODPROC
		#define F90MODPROC(a, b)    CXXFUNC(b, b)
	#endif
#endif



//
//	RLUX: returns a vector RVEC of LEN 32-bit random floating point numbers
//		  between zero (not included) and one (also not incl.)
//
PROTOCCALLSFSUB2(F90MODPROC(slatec, ranlux), ranlux,					\
                 FLOATV, INT)
#define RANLUX_RLUX(RVEC, LEN)											\
    CCALLSFSUB2(RANLUX, ranlux, FLOATV, INT, RVEC, LEN)

//
//  RLUXGO: initializes the generator from one 32-bit integer INT and sets
//          Luxury Level
//
PROTOCCALLSFSUB4(F90MODPROC(slatec, rluxgo), rluxgo,					\
                 INT, INT, INT, INT)
#define RANLUX_RLUXGO(LUX, SEED)                                        \
    CCALLSFSUB4(RLUXGO, rluxgo, INT, INT, INT, INT, LUX, SEED, 0, 0)


static Ranlux *ranluxInst = 0;	// single global static instance of Ranlux


//
//	methods
//
Ranlux::Ranlux(SophisticationLevel level, unsigned seed, double min,
			   double max)
			: RandomNumberGenerator(level, seed, min, max), _range(max-min)
{
	PROC

	if (!seed) {
		srand((unsigned)std::time((time_t *)NULL));
		seed = std::rand();
	}

	_seed = seed;
	RANLUX_RLUXGO(UNT(_level), _seed);

	ENDPROC
} // Ranlux::Ranlux()


void Ranlux::sophisticationLevel(SophisticationLevel level)
{
	PROC

	_level = level;
	reset();

	ENDPROC
} // Ranlux::sophisticationLevel()


void Ranlux::seed(unsigned seed)
{
	PROC

	_seed = seed;
	reset();

	ENDPROC
} // Ranlux::seed()


void Ranlux::min(double min)
{
	PROC

	_min = min;
	_range = _max - min;
} // Ranlux::min()


void Ranlux::max(double max)
{
	PROC

	_max = max;
	_range = max - _min;
} // Ranlux::min()


void Ranlux::reset()
{
	PROC

	unsigned seed = _seed;
	SophisticationLevel level = _level;
	double min = _min, max = _max;
	delete ranluxInst;
	ranluxInst = new Ranlux(level, seed, min, max);

	ENDPROC
} // Ranlux::reset()


double Ranlux::rand()
{
	PROC

	const unsigned RNDNUM_BUFF_LEN = 100; // length of buffer for rnd numbers

	static float rndNumBuff[RNDNUM_BUFF_LEN];
	static unsigned buffIndx = RNDNUM_BUFF_LEN;

	if (buffIndx < RNDNUM_BUFF_LEN) {
		//
		//	still unused numbers in buffer
		//
		ALTRETURNV(_range * DBL(rndNumBuff[buffIndx++]) + _min);
	}

	//
	//	re-fill the buffer
	//
	RANLUX_RLUX(rndNumBuff, RNDNUM_BUFF_LEN);
	buffIndx = 0;

	RETURN this->rand();
} // Ranlux::rand()


RandomNumberGenerator &randGenFunc()
{
	PROC

	static bool first = true;
	if (first) {
		first = false;
		ranluxInst = new Ranlux;
	}

	RETURN *ranluxInst;
} // randGenFunc()
