//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                   (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	Ranlux.h:	--- Conrete implementation of RandomNumberGenerator class
//
//	DESCRIPTION:
//		Class Ranlux provides an implementation of the abstract class
//		RandomNumberGenerator in Random.h. It is based in the Fortran
//		subroutine RANLUX in ranlux.f:
//		Subtract-and-borrow random number generator proposed by
//		Marsaglia and Zaman, implemented by F. James with the name
//		RCARRY in 1991, and later improved by Martin Luescher
//		in 1993 to produce "Luxury Pseudorandom Numbers".
//		Fortran 77 coded by F. James, 1993
//
//		M. Luscher, Computer Physics Communications  79 (1994) 100
//		F. James, Computer Physics Communications 79 (1994) 111
//
//	CHANGE HISTORY:
//		UL, 1999-05-27: 1.0.0: - first version
//		UL, 1999-06-08: 1.1.0: - added min()/max() methods
//
//	AUTHOR:
//		U. Lammers (UL), Thu May 27 16:15:23 MET DST 1999
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/utils/src/Ranlux.h,v 1.2 1999/06/08 16:08:40 ulammers Exp $
//

#ifndef RANLUX_H
#define RANLUX_H

#include "Random.h"

class Ranlux : public virtual RandomNumberGenerator {
public:
	Ranlux(SophisticationLevel level = ALMOST_PERFECT, unsigned seed = 0,
		   double min = 0., double max = 1.);
		//
		//	PURPOSE:
		//		Construct a Ranlux object
		//
		//	ARGUMENTS:
		//		
		//		In:
		//			level	: Describes the level of sophistication
		//					  of the generator in terms of quality of the
		//					  returned random numbers, the following mapping
		//					  applies:
		//						QUICK_AND_DIRTY:
		//							(p=24): equivalent to the original RCARRY
		//									of Marsaglia and Zaman, very long
		//									period, but fails many tests.
		//						GOOD:
		//							(p=48): considerable improvement in
		//									quality over level 0, now passes
		//									the gap test, but still fails
		//									spectral test.
		//						REALLY_GOOD:
		//							(p=97): passes all known tests, but
		//									theoretically still defective.
		//						ALMOST_PERFECT:
		//							(p=223): DEFAULT VALUE.  Any theoretically
		//									 possible correlations have very
		//									 small chance of being observed.
		//						
		//						PERFECT:
		//							(p=389): highest possible luxury, all 24
		//									 bits chaotic.
		//			seed	: see Random.h
		//			min/max	: lower/upper bound of returned numbers
		//

	void sophisticationLevel(SophisticationLevel level);
	void seed(unsigned seed);
		//
		//	PURPOSE:
		//		See Random.h
		//

	void min(double min);
	void max(double max);
		//
		//	PURPOSE:
		//		See Random.h
		//		

	void reset();
		//
		//	PURPOSE:
		//		See Random.h
		//

	double rand();
		//
		//	PURPOSE:
		//		See Random.h

	virtual ~Ranlux() { PROC ENDPROC }
		//
		//	PURPOSE:
		//		See Random.h
		//

private:
	double	_range;
};

#endif
